﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.ComponentModel;
using System.Xml;

namespace DxDesigner.Components
{
    [Designer(typeof(DxContainerDesigner))]
    class DxContainer:UserControl
    {
        public enum _rotate { deg0, deg90, deg180, deg270 };
        private void InitializeComponent()
        {
            this.SuspendLayout();
            // 
            // DxContainer
            // 
            this.AllowDrop = true;
            this.Name = "DxContainer";
            this.Size = new System.Drawing.Size(588, 368);
            this.ResumeLayout(false);

        }

        bool swl = false;
        [Description("Template for SWL confirmations?"),Category("Template")]
        public bool SWL
        {
            set
            {
                swl = value;
            }
            get
            {
                return swl;
            }
        }

        float width_mm = 140.0f;
        [Description("Width of QSL [mm]"), Category("Template")]
        public float Width_mm
        {
            set
            {
                width_mm = value;
                Width = (int)(value * MainForm.sizeconst);
            }
            get
            {
                return width_mm;
            }
        }

        float height_mm = 90.0f;
        [Description("Height of QSL [mm]"), Category("Template")]
        public float Height_mm
        {
            set
            {
                height_mm = value;
                Height = (int)(value * MainForm.sizeconst);
            }
            get
            {
                return height_mm;
            }
        }

        _rotate rotate = _rotate.deg0;
        [Description("Rotation of QSL card in printer"), Category("Template")]
        public _rotate Rotate
        {
            set
            {
                rotate = value;
            }
            get
            {
                return rotate;
            }
        }

        public XmlNode DxSerialize(XmlDocument parent)
        {
            XmlNode comp = parent.CreateElement("template");
            try
            {
                XmlNode param = parent.CreateElement("pageHeight");
                XmlNode val = parent.CreateTextNode((((rotate == _rotate.deg0) || (rotate == _rotate.deg180) ? Height_mm : Width_mm)).ToString());
                param.AppendChild(val);
                comp.AppendChild(param);

                param = parent.CreateElement("pageWidth");
                val = parent.CreateTextNode((((rotate == _rotate.deg0) || (rotate == _rotate.deg180) ? Width_mm : Height_mm)).ToString());
                param.AppendChild(val);
                comp.AppendChild(param);

                param = parent.CreateElement("rotate");
                int deg = 0;
                switch (rotate)
                {
                    case _rotate.deg90:
                        deg = 90;
                        break;
                    case _rotate.deg180:
                        deg = 180;
                        break;
                    case _rotate.deg270:
                        deg = 270;
                        break;
                    default:
                        deg = 0;
                        break;
                }
                val = parent.CreateTextNode(deg.ToString());
                param.AppendChild(val);
                comp.AppendChild(param);

                param = parent.CreateElement("SWL");
                val = parent.CreateTextNode(swl?"1":"0");
                param.AppendChild(val);
                comp.AppendChild(param);

            }
            catch
            {
            }
            return comp;
        }

        public void DxDeserialize(XmlNode node)
        {
            int ww = 140;
            int hh = 90;
            int rr = 0;
            while (node != null)
            {
                switch (node.Name.ToLower())
                {
                    case "swl":
                        swl = node.FirstChild.Value != "0";
                        break;
                    case "pagewidth":
                        ww = (int)float.Parse(node.FirstChild.Value); ;
                        break;
                    case "pageheight":
                        hh = (int)float.Parse(node.FirstChild.Value); ;
                        break;
                    case "rotate":
                        rr = int.Parse(node.FirstChild.Value);
                        break;
                }
                node = node.NextSibling;
            }

            BackColor = System.Drawing.Color.White;
            BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
            bool rot = false;
            switch (rr)
            {
                case 0:
                    rotate = _rotate.deg0;
                    break;
                case 90:
                    rotate = _rotate.deg90;
                    rot = true;
                    break;
                case 180:
                    rotate = _rotate.deg180;
                    break;
                case 270:
                    rotate = _rotate.deg270;
                    rot = true;
                    break;
            }
            Width_mm = rot ? hh : ww;
            Height_mm = rot ? ww : hh;
        }
    }
}
