﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Xml;

namespace DxDesigner.Components
{
    /// <summary>
    /// Designer component representing date and time database entry
    /// </summary>
    [Designer(typeof(DxTextComponentDesigner))]
    public partial class DxDateTime : System.Windows.Forms.Label
    {
        public DxDateTime()
        {
            this.dtformat = "dd MMM yyyy HH:mm";
            InitializeComponent();
        }

        int index = 1;
        [Category("DxDateTime"), Description("Number of selected QSO record (1 to 3)"), DefaultValue(0)]
        public int Index
        {
            set
            {
                index = value;
            }
            get
            {
                return index;
            }
        }

        string dtformat;
        [Category("DxDateTime"), Description("Formating like\r\ndd MMM yyyy HH:mm"), DefaultValue("dd MMM yyyy HH:mm")]
        public string DTFormat
        {
            set
            {
                dtformat = value;
            }
            get
            {
                return dtformat;
            }
        }

        public readonly DateTime testDate = new DateTime(2011, 12, 24, 20, 00, 00);

        public XmlNode DxSerialize(XmlDocument parent)
        {
            XmlNode comp = parent.CreateElement("datetime");
            try
            {
                XmlNode param = parent.CreateElement("x");
                XmlNode val = parent.CreateTextNode((this.Location.X / MainForm.sizeconst).ToString());
                param.AppendChild(val);
                comp.AppendChild(param);

                param = parent.CreateElement("y");
                val = parent.CreateTextNode((this.Location.Y / MainForm.sizeconst).ToString());
                param.AppendChild(val);
                comp.AppendChild(param);

                param = parent.CreateElement("index");
                val = parent.CreateTextNode(this.index.ToString());
                param.AppendChild(val);
                comp.AppendChild(param);

                param = parent.CreateElement("DTFormat");
                val = parent.CreateTextNode(this.DTFormat);
                param.AppendChild(val);
                comp.AppendChild(param);

                if (this.Font.FontFamily.Name != MainForm.defaultFont)
                {
                    param = parent.CreateElement("font");
                    val = parent.CreateTextNode(this.Font.FontFamily.Name);
                    param.AppendChild(val);
                    comp.AppendChild(param);
                }

                if (this.Font.Style != FontStyle.Regular)
                {
                    param = parent.CreateElement("style");
                    val = parent.CreateTextNode(this.Font.Style.ToString());
                    param.AppendChild(val);
                    comp.AppendChild(param);
                }

                if (this.Font.Size != MainForm.defaultFontSize)
                {
                    param = parent.CreateElement("size");
                    val = parent.CreateTextNode(this.Font.Size.ToString());
                    param.AppendChild(val);
                    comp.AppendChild(param);
                }

                if (this.ForeColor.ToKnownColor() != System.Drawing.KnownColor.ControlText)
                {
                    param = parent.CreateElement("color");
                    val = parent.CreateTextNode(this.ForeColor.Name);
                    param.AppendChild(val);
                    comp.AppendChild(param);
                }
            }
            catch
            {
            }

            return comp;
        }

        public void DxDeserialize(XmlNode node)
        {
            string family = MainForm.defaultFont;
            float size = MainForm.defaultFontSize;
            FontStyle style = FontStyle.Regular;
            this.DTFormat = "dd MMM yyyy HH:mm";

            while (node != null)
            {
                switch (node.Name)
                {
                    case "x":
                        Left = (int)(MainForm.sizeconst * float.Parse(node.FirstChild.Value)); ;
                        break;
                    case "y":
                        Top = (int)(MainForm.sizeconst * float.Parse(node.FirstChild.Value));
                        break;
                    case "index":
                        index = int.Parse(node.FirstChild.Value);
                        break;
                    case "font":
                        family = node.FirstChild.Value;
                        break;
                    case "size":
                        size = float.Parse(node.FirstChild.Value);
                        break;
                    case "style":
                        string val = node.FirstChild.Value;
                        if (val.IndexOf("Bold") >= 0)
                            style |= FontStyle.Bold;
                        if (val.IndexOf("Underline") >= 0)
                            style |= FontStyle.Underline;
                        if (val.IndexOf("Italic") >= 0)
                            style |= FontStyle.Italic;
                        break;
                    case "color":
                        this.ForeColor = Color.FromName(node.FirstChild.Value);
                        break;
                    case "DTFormat":
                        if (node.FirstChild != null)
                        {
                            this.DTFormat = node.FirstChild.Value;
                        }
                        break;
                }
                node = node.NextSibling;
            }
            Text = testDate.ToString(DTFormat);
            Font = new Font(family, size, style);

        }
    }
}
