﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Xml;

namespace DxDesigner.Components
{
    [Designer(typeof(DxTextComponentDesigner))]
    public partial class DxLabel : System.Windows.Forms.Label
    {
        public DxLabel()
        {
            InitializeComponent();
            Text = "???";
        }

        [Category("DxText")]
        public string text
        {
            set
            {
                Text = value;
            }
            get
            {
                return Text;
            }

        }

        public XmlNode DxSerialize(XmlDocument parent)
        {
            XmlNode comp = parent.CreateElement("label");
            try
            {
                XmlNode param = parent.CreateElement("x");
                XmlNode val = parent.CreateTextNode((this.Location.X / MainForm.sizeconst).ToString());
                param.AppendChild(val);
                comp.AppendChild(param);

                param = parent.CreateElement("y");
                val = parent.CreateTextNode((this.Location.Y / MainForm.sizeconst).ToString());
                param.AppendChild(val);
                comp.AppendChild(param);

                param = parent.CreateElement("text");
                val = parent.CreateTextNode(this.text);
                param.AppendChild(val);
                comp.AppendChild(param);

                if (this.Font.FontFamily.Name != MainForm.defaultFont)
                {
                    param = parent.CreateElement("font");
                    val = parent.CreateTextNode(this.Font.FontFamily.Name);
                    param.AppendChild(val);
                    comp.AppendChild(param);
                }

                if (this.Font.Style != FontStyle.Regular)
                {
                    param = parent.CreateElement("style");
                    val = parent.CreateTextNode(this.Font.Style.ToString());
                    param.AppendChild(val);
                    comp.AppendChild(param);
                }

                if (this.Font.Size != MainForm.defaultFontSize)
                {
                    param = parent.CreateElement("size");
                    val = parent.CreateTextNode(this.Font.Size.ToString());
                    param.AppendChild(val);
                    comp.AppendChild(param);
                }

                if (this.ForeColor.ToKnownColor() != System.Drawing.KnownColor.ControlText)
                {
                    param = parent.CreateElement("color");
                    val = parent.CreateTextNode(this.ForeColor.Name);
                    param.AppendChild(val);
                    comp.AppendChild(param);
                }
            }
            catch
            {
            }

            return comp;
        }

        public void DxDeserialize(XmlNode node)
        {
            string family = MainForm.defaultFont;
            float size = MainForm.defaultFontSize;
            FontStyle style = FontStyle.Regular;
            while (node != null)
            {
                if (node.FirstChild != null)
                {
                    switch (node.Name)
                    {
                        case "x":
                            Left = (int)(MainForm.sizeconst * float.Parse(node.FirstChild.Value));
                            break;
                        case "y":
                            Top = (int)(MainForm.sizeconst * float.Parse(node.FirstChild.Value));
                            break;
                        case "text":
                            text = node.FirstChild.Value;
                            break;
                        case "font":
                            family = node.FirstChild.Value;
                            break;
                        case "size":
                            size = float.Parse(node.FirstChild.Value);
                            break;
                        case "style":
                            string val = node.FirstChild.Value;
                            if (val.IndexOf("Bold") >= 0)
                                style |= FontStyle.Bold;
                            if (val.IndexOf("Underline") >= 0)
                                style |= FontStyle.Underline;
                            if (val.IndexOf("Italic") >= 0)
                                style |= FontStyle.Italic;
                            break;
                        case "color":
                            this.ForeColor = Color.FromName(node.FirstChild.Value);
                            break;
                    }
                }
                node = node.NextSibling;
            }
            Font = new Font(family, size, style);

        }
    }

}
