﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Xml;

namespace DxDesigner.Components
{
    [Designer(typeof(DxTextComponentDesigner))]
    public partial class DxText : System.Windows.Forms.Label
    {
        public enum dbColumns { Call, Rst, Mode, Qth, Manager };

        public DxText()
        {
            InitializeComponent();
            Column = dbColumns.Call;
        }

        int index = 1;
        [Category("DxText"), Description("Number of selected QSO record"), DefaultValue(0)]
        public int Index
        {
            set
            {
                index = value;
            }
            get
            {
                return index;
            }
        }

        dbColumns column = dbColumns.Call;
        [Category("DxNumber"), Description("Database column name")]
        public dbColumns Column
        {
            set
            {
                column = value;
                switch (value)
                {
                    case dbColumns.Call:
                        Text = "OK1DX/mm";
                        break;
                    case dbColumns.Mode:
                        Text = "RTTY";
                        break;
                    case dbColumns.Qth:
                        Text = "JN69SD";
                        break;
                    case dbColumns.Rst:
                        Text = "599";
                        break;
                    case dbColumns.Manager:
                        Text = "W3HNK";
                        break;
                }

            }
            get
            {
                return column;
            }
        }

        public XmlNode DxSerialize(XmlDocument parent)
        {
            XmlNode comp = parent.CreateElement("text");
            try
            {
                XmlNode param = parent.CreateElement("x");
                XmlNode val = parent.CreateTextNode((this.Location.X / MainForm.sizeconst).ToString());
                param.AppendChild(val);
                comp.AppendChild(param);

                param = parent.CreateElement("y");
                val = parent.CreateTextNode((this.Location.Y / MainForm.sizeconst).ToString());
                param.AppendChild(val);
                comp.AppendChild(param);

                param = parent.CreateElement("column");
                val = parent.CreateTextNode(this.column.ToString());
                param.AppendChild(val);
                comp.AppendChild(param);

                param = parent.CreateElement("index");
                val = parent.CreateTextNode(this.index.ToString());
                param.AppendChild(val);
                comp.AppendChild(param);

                if (this.Font.FontFamily.Name != MainForm.defaultFont)
                {
                    param = parent.CreateElement("font");
                    val = parent.CreateTextNode(this.Font.FontFamily.Name);
                    param.AppendChild(val);
                    comp.AppendChild(param);
                }

                if (this.Font.Style != FontStyle.Regular)
                {
                    param = parent.CreateElement("style");
                    val = parent.CreateTextNode(this.Font.Style.ToString());
                    param.AppendChild(val);
                    comp.AppendChild(param);
                }

                if (this.Font.Size != MainForm.defaultFontSize)
                {
                    param = parent.CreateElement("size");
                    val = parent.CreateTextNode(this.Font.Size.ToString());
                    param.AppendChild(val);
                    comp.AppendChild(param);
                }

                if (this.ForeColor.ToKnownColor() != System.Drawing.KnownColor.ControlText)
                {
                    param = parent.CreateElement("color");
                    val = parent.CreateTextNode(this.ForeColor.Name);
                    param.AppendChild(val);
                    comp.AppendChild(param);
                }
            }
            catch
            {
            }

            return comp;
        }

        public void DxDeserialize(XmlNode node)
        {
            string family = MainForm.defaultFont;
            float size = MainForm.defaultFontSize;
            FontStyle style = FontStyle.Regular;
            while (node != null)
            {
                switch (node.Name.ToLower())
                {
                    case "x":
                        Left = (int)(MainForm.sizeconst * float.Parse(node.FirstChild.Value)); ;
                        break;
                    case "y":
                        Top = (int)(MainForm.sizeconst * float.Parse(node.FirstChild.Value)); ;
                        break;
                    case "column":
                        switch (node.FirstChild.Value.ToLower())
                        {
                            case "qth":
                                Column = DxText.dbColumns.Qth;
                                break;
                            case "call":
                                Column = DxText.dbColumns.Call;
                                break;
                            case "mode":
                                Column = DxText.dbColumns.Mode;
                                break;
                            case "rst":
                                Column = DxText.dbColumns.Rst;
                                break;
                            case "manager":
                                Column = DxText.dbColumns.Manager;
                                break;
                        }
                        break;
                    case "index":
                        Index = int.Parse(node.FirstChild.Value);
                        break;
                    case "font":
                        family = node.FirstChild.Value;
                        break;
                    case "size":
                        size = float.Parse(node.FirstChild.Value);
                        break;
                    case "style":
                        string val = node.FirstChild.Value;
                        if (val.IndexOf("Bold") >= 0)
                            style |= FontStyle.Bold;
                        if (val.IndexOf("Underline") >= 0)
                            style |= FontStyle.Underline;
                        if (val.IndexOf("Italic") >= 0)
                            style |= FontStyle.Italic;
                        break;
                    case "color":
                        this.ForeColor = Color.FromName(node.FirstChild.Value);
                        break;
                }
                node = node.NextSibling;
            }
            Font = new Font(family, size, style);
        }
    }
}
