﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel.Design;
using System.ComponentModel.Design.Serialization;
using System.Collections;
using DxDesigner.Components;
using System.Xml;
using System.Drawing;

namespace DxDesigner
{
    class DxSerializationService : IDesignerSerializationService
    {
        IDesignerHost host = null;
        public DxSerializationService(IDesignerHost host)
        {
            this.host = host;
        }

        /// <summary>
        /// Serializes the collection of objects into data object
        /// Data object is in fact collection of individual XML strings describing components
        /// Used for Copy/Paste
        /// </summary>
        /// <param name="objects"></param>
        /// <returns></returns>
        public object Serialize(ICollection objects)
        {
            XmlDocument doc = new XmlDocument();
            ArrayList elems = new ArrayList();
            foreach (object o in objects)
            {
                if (o is string[])
                {
                    XmlNode node = doc.CreateElement("strings");
                    foreach (string str in (string[])o)
                    {
                        XmlNode snode = doc.CreateElement("string");
                        XmlNode val = doc.CreateTextNode(str);
                        snode.AppendChild(val);
                        node.AppendChild(snode);
                    }
                    elems.Add(node.OuterXml);
                }
                else if (o is DxFrame)
                {
                    DxFrame frame = o as DxFrame;
                    XmlNode node = frame.DxSerialize(doc);
                    elems.Add(node.OuterXml);
                }
                else if (o is DxLabel)
                {
                    DxLabel label = o as DxLabel;
                    XmlNode node = label.DxSerialize(doc);
                    elems.Add(node.OuterXml);
                }
                else if (o is DxSwlCall)
                {
                    DxSwlCall swlcall = o as DxSwlCall;
                    XmlNode node = swlcall.DxSerialize(doc);
                    elems.Add(node.OuterXml);
                }
                else if (o is DxText)
                {
                    DxText dxtext = o as DxText;
                    XmlNode node = dxtext.DxSerialize(doc);
                    elems.Add(node.OuterXml);
                }
                else if (o is DxNumber)
                {
                    DxNumber dxnumber = o as DxNumber;
                    XmlNode node = dxnumber.DxSerialize(doc);
                    elems.Add(node.OuterXml);
                }
                else if (o is DxDateTime)
                {
                    DxDateTime datetime = o as DxDateTime;
                    XmlNode node = datetime.DxSerialize(doc);
                    elems.Add(node.OuterXml);
                }
                else if (o is DxPicture)
                {
                    DxPicture picture = o as DxPicture;
                    XmlNode node = picture.DxSerialize(doc);
                    elems.Add(node.OuterXml);
                }
            }
            return elems;
        }


        /// <summary>
        /// Deserializes data object into collection of components.
        /// Used for Copy/Paste 
        /// </summary>
        /// <param name="serializationData"></param>
        /// <returns></returns>
        public ICollection Deserialize(object serializationData)
        {
            ArrayList col = new ArrayList();
            ArrayList data = serializationData as ArrayList;
            XmlDocument doc = new XmlDocument();
            if (data != null)
            {
                foreach (object d in data)
                {
                    doc.LoadXml(d as string);
                    switch (doc.DocumentElement.Name)
                    {
                        case "strings":
                            #region
                            string[] ss = new string[doc.DocumentElement.ChildNodes.Count];
                            int i = 0;
                            XmlNode sn = doc.DocumentElement.FirstChild;
                            while (sn != null)
                            {
                                ss[i++] = sn.FirstChild == null ? "" : sn.FirstChild.Value;
                                sn = sn.NextSibling;
                            }
                            col.Add(ss);
                            break;
                            #endregion
                        case "frame":
                            DxFrame frame = (DxFrame)host.CreateComponent(typeof(DxFrame));
                            frame.DxDeserialize(doc.DocumentElement.FirstChild);
                            col.Add(frame);
                            break;
                        case "label":
                            DxLabel label = (DxLabel)host.CreateComponent(typeof(DxLabel));
                            label.DxDeserialize(doc.DocumentElement.FirstChild);
                            col.Add(label);
                            break;
                        case "swlcall":
                            DxSwlCall swlcall = (DxSwlCall)host.CreateComponent(typeof(DxSwlCall));
                            swlcall.DxDeserialize(doc.DocumentElement.FirstChild);
                            col.Add(swlcall);
                            break;
                        case "text":
                            DxText dxtext = (DxText)host.CreateComponent(typeof(DxText));
                            dxtext.DxDeserialize(doc.DocumentElement.FirstChild);
                            col.Add(dxtext);
                            break;
                        case "number":
                            DxNumber dxnumber = (DxNumber)host.CreateComponent(typeof(DxNumber));
                            dxnumber.DxDeserialize(doc.DocumentElement.FirstChild);
                            col.Add(dxnumber);
                            break;
                        case "datetime":
                            DxDateTime datetime = (DxDateTime)host.CreateComponent(typeof(DxDateTime));
                            datetime.DxDeserialize(doc.DocumentElement.FirstChild);
                            col.Add(datetime);
                            break;
                        case "picture":
                            DxPicture picture = (DxPicture)host.CreateComponent(typeof(DxPicture));
                            picture.DxDeserialize(doc.DocumentElement.FirstChild);
                            col.Add(picture);
                            break;
                    }
                }
            }
            return col;
        }
    }
}
