using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Design;
using System.Text;
using System.Windows.Forms;
using System.ComponentModel.Design;
using DxDesigner.Host;
using DxDesigner.Components;
using System.ComponentModel.Design.Serialization;
using System.Diagnostics;
using System.IO;

namespace DxDesigner
{
    public partial class MainForm : Form
    {
        public const float sizeconst = 100.0f / 25.4f;
        const string dxqsl = "\\DxQsl.exe";
        const string banner = "DxQSL template designer";
        public const string defaultFont = "Arial";
        public const float defaultFontSize = 10.0f;

        public static string fileName = ""; // full path name of QSL template loaded
        DxHostControl dxhc = null;

        public MainForm()
        {
            InitializeComponent();
            if (File.Exists(Application.StartupPath + dxqsl))
                startDxQslToolStripMenuItem.Enabled = true;
            CustomInitialize();
        }

        /// <summary>
        /// Create instance of design surface manager; basic services added
        /// </summary>
        private void CustomInitialize()
        {
            dxDesignSurfaceManager = new DxDesignSurfaceManager();
            dxDesignSurfaceManager.AddService(typeof(IToolboxService), this.toolbox1);  // toolbox connected to manager
            dxDesignSurfaceManager.AddService(typeof(System.Windows.Forms.PropertyGrid), this.propertyGrid1);   // property grid connected
        }

        /// <summary>
        /// Loads the QSL design from QSL template file
        /// </summary>
        private void LoadQSLdesign()
        {
            ContPanel.Controls.Clear();
            DxHostControl hc = dxDesignSurfaceManager.GetNewHost();
            dxhc = hc;
            hc.Parent = ContPanel;
            hc.Dock = DockStyle.Fill;
            toolbox1.DesignerHost = hc.DesignerHost;
            Text = banner;
            if (fileName.Length > 0)
                Text += " - " + fileName;
        }

        /// <summary>
        /// Common event handler for most of menu commands
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuAction(object sender, EventArgs e)
        {
            string command = sender.ToString();
            IMenuCommandService mcs = (IMenuCommandService)dxhc.DesignSurface.GetService(typeof(IMenuCommandService));
            switch (command.ToLower())
            {
                case "delete":
                    mcs.GlobalInvoke(StandardCommands.Delete);
                    break;
                case "copy":
                    mcs.GlobalInvoke(StandardCommands.Copy);
                    break;
                case "paste":
                    mcs.GlobalInvoke(StandardCommands.Paste);
                    break;
                case "bring to front":
                    mcs.GlobalInvoke(StandardCommands.BringToFront);
                    break;
                case "send to back":
                    mcs.GlobalInvoke(StandardCommands.SendToBack);
                    break;
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void openTemplateToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Filter = "template file (*.xml)|*.xml";
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                fileName = dlg.FileName;
                LoadQSLdesign();
            }
        }

        private void saveTemplateToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveFileDialog dlg = new SaveFileDialog();
            dlg.Filter = "template file (*.xml)|*.xml";
            dlg.FileName = fileName;
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                fileName = dlg.FileName;
                DxDesignerLoader dl = (DxDesignerLoader)dxhc.DesignSurface.Loader;
                dl._Modified = true;
                dxhc.DesignSurface.Flush();
            }
            Text = banner;
            if (fileName.Length > 0)
                Text += " - " + fileName;
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            try
            {
                DxDesignerLoader dl = (DxDesignerLoader)dxhc.DesignSurface.Loader;
                if (dl._Modified)
                {
                    DialogResult dr = MessageBox.Show("Template has been modified, save?", "Warning", MessageBoxButtons.YesNoCancel);
                    if (dr == DialogResult.Yes)
                        dxhc.DesignSurface.Flush();
                    else if (dr == DialogResult.Cancel)
                        e.Cancel = true;
                }
            }
            catch { }
        }

        private void newTemplateToolStripMenuItem_Click(object sender, EventArgs e)
        {
            fileName = "";
            LoadQSLdesign();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Dx Template Designer 1.0.0\r\nFor more information\r\nsee http://ok1dx.cz/.\r\n\r\n73 Pavel OK1DX", "About");
        }

        private void startDxQslToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                Process.Start(Application.StartupPath + dxqsl);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error starting DxQSL");
            }
        }

        private void propertyGrid1_Enter(object sender, EventArgs e)
        {
            deleteToolStripMenuItem.ShortcutKeys = Keys.None;
        }

        private void propertyGrid1_Leave(object sender, EventArgs e)
        {
            deleteToolStripMenuItem.ShortcutKeys = Keys.Delete;
        }

    }
}