﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Printing;
using System.Xml;
using System.Globalization;
using System.Threading;
using System.IO;
using System.Xml.Serialization;

namespace DxQsl
{
    public partial class DxQslDlg : Form
    {
        public const float cc = 100.0f / 25.4f; // convertion from milimeters to centiinches
        const string prnSettingFileName = "DxQsl.prn";

        string[] bandnames = { "160m", "80m", "75m", "40m", "30m", "20m", "17m", "15m", "12m", "10m", "6m", "4m","2m", "70cm" };
        double[] bandfreq = { 1.830, 3.5, 3.8, 7.0, 10.1, 14.0, 18.1, 21.0, 24.9, 28.0, 50.0, 70.0, 145.0, 432.0 };
        
        PrintDocument doc = null;
        XmlDocument templ = null;
        DataRow adifQso = null;
        string adifdate, adiftime;
        int qsocount;
        string templatePath;
        string templFile;
        string adifFile;
        bool swlMode = false;
        public static float xshift = 0;
        public static float yshift = 0;
        private DxQsl.Properties.Settings settings = new DxQsl.Properties.Settings();
        
        public DxQslDlg()
        {
            CultureInfo objCI = new CultureInfo(0x007F);
            objCI.NumberFormat.NumberDecimalSeparator = ".";
            //objCI.DateTimeFormat.MonthNames = new string[] {"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec",""};
            objCI.DateTimeFormat.ShortDatePattern = "dd.MM.yyyy";
            //objCI.DateTimeFormat.DateSeparator = ".";
            Thread.CurrentThread.CurrentCulture = objCI;
            Thread.CurrentThread.CurrentUICulture = objCI;
            
            InitializeComponent();
            doc = new PrintDocument();
            doc.PrintPage += new PrintPageEventHandler(doc_PrintPage);
            txtSearchCall.Select();
            this.Text = "DxQSL ver. " + Application.ProductVersion;

            if (File.Exists(prnSettingFileName))
            {
                XmlSerializer xs = new XmlSerializer(typeof(PrinterSettings));
                StreamReader sr = new StreamReader(prnSettingFileName);
                doc.PrinterSettings = (PrinterSettings)xs.Deserialize(sr);
                sr.Close();
            }
        }

        void LoadPrinterSettings()
        {
            float pageWidth = 140;
            float pageHeight = 90;
            XmlNode node = templ.DocumentElement.FirstChild;
            while (node != null)
            {
                if (node.Name == "template")
                {
                    XmlNode xnode = node.FirstChild;
                    while (xnode != null)   // faceplate parameters
                    {
                        switch (xnode.Name)
                        {
                            case "pageWidth":
                                pageWidth = (float)double.Parse(xnode.InnerText);
                                break;
                            case "pageHeight":
                                pageHeight = (float)double.Parse(xnode.InnerText);
                                break;
                            case "SWL":
                                swlMode = (xnode.InnerText == "1") || (xnode.InnerText == "Y");
                                break;
                        }
                        xnode = xnode.NextSibling;
                    }
                }
                node = node.NextSibling;
            }
            PaperSize ps = new PaperSize("Custom", (int)(pageWidth * cc), (int)(pageHeight * cc));
            doc.DefaultPageSettings.PaperSize = ps;

            // txtSwl.Text = "";
            txtSwl.Visible = swlMode;
            label2.Visible = swlMode;

            //ps.Kind = PaperKind.Custom;
            //ps.Height = (int)(pageHeight * cc);
            //ps.Width = (int)(pageWidth * cc);
            //doc.DefaultPageSettings.PaperSize.Kind = PaperKind.Custom;
            //PaperSize ps = new PaperSize("Custom", (int)(pageWidth * cc), (int)(pageHeight * cc));

        }

        void  doc_PrintPage(object sender, PrintPageEventArgs e)
        {
            float width = 100;
            float height = 100;
            int rotate = 0;
            xshift = 0;
            yshift = 0;
            
            if (templ == null)
                return;
            XmlNode node = templ.DocumentElement.FirstChild;
            while (node != null)
            {
                if (node.Name == "template")
                {
                    XmlNode xnode = node.FirstChild;
                    while (xnode != null)   // faceplate parameters
                    {
                        switch (xnode.Name)
                        {
                            case "pageWidth":
                                width = (float)double.Parse(xnode.InnerText);
                                break;
                            case "pageHeight":
                                height = (float)double.Parse(xnode.InnerText);
                                break;
                            case "rotate":
                                rotate = int.Parse(xnode.InnerText);
                                rotate = (rotate / 90) * 90;
                                while (rotate >= 360)
                                    rotate -= 360;
                                while (rotate < 0)
                                    rotate += 360;
                                break;
                            case "xshift":
                                xshift = (float)double.Parse(xnode.InnerText);
                                break;
                            case "yshift":
                                yshift = (float)double.Parse(xnode.InnerText);
                                break;

                        }
                        xnode = xnode.NextSibling;
                    }
                    switch(rotate)
                    {
                        case 90:
                            e.Graphics.TranslateTransform(width*cc, 0);
                            break;
                        case 180:
                            e.Graphics.TranslateTransform(width*cc, height*cc);
                            break;
                        case 270:
                            e.Graphics.TranslateTransform(0, height * cc);
                            break;
                    }
                    e.Graphics.RotateTransform(rotate);



                    xnode = node.FirstChild;
                    while (xnode != null)   // objects
                    {
                        DxBase elem = null;
                        switch (xnode.Name.ToLower())
                        {
                            case "label":
                                elem = new DxLabel(xnode);
                                break;
                            case "frame":
                                elem = new DxFrame(xnode);
                                break;
                            case "text":
                                elem = new DxText(xnode,dataSet1.Tables[0]);
                                break;
                            case "datetime":
                                elem = new DxDateTime(xnode, dataSet1.Tables[0]);
                                break;
                            case "number":
                                elem = new DxNumber(xnode, dataSet1.Tables[0]);
                                break;
                            case "picture":
                                elem = new DxPicture(xnode, templatePath);
                                break;
                            case"swlcall":
                                elem = new DxSwlCall(xnode, txtSwl.Text);
                                break;
                        }
                        if (elem != null)
                            elem.Print(e.Graphics, 0.0f, 0.0f);

                        xnode = xnode.NextSibling;
                    }
                }
                node = node.NextSibling;
            }
        }

        private void butPreview_Click(object sender, EventArgs e)
        {
            LoadTemplate(templ == null);

            if (printGrid.Rows.Count <= 1)
            {
                MessageBox.Show("No QSO to be printed. Double click the log entry!", "Error");
                return;
            }

            try
            {
                PrintPreviewDialog dlgPreview = new PrintPreviewDialog();
                dlgPreview.Width = 600;
                dlgPreview.Height = 800;
                dlgPreview.Document = doc;
                dlgPreview.ShowDialog();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error");
            }
        }

        private void butPrint_Click(object sender, EventArgs e)
        {
            if (templ == null)
                LoadTemplate(true);

            if (printGrid.Rows.Count <= 1)
            {
                MessageBox.Show("No QSO to be printed. Double click the log entry!", "Error");
                return;
            }
            doc.Print();
            dataSet1.Tables[0].Rows.Clear();
            txtSearchCall.SelectionStart = 0;
            txtSearchCall.SelectionLength = 100;
            txtSearchCall.Focus();
        }

        private void butSetup_Click(object sender, EventArgs e)
        {
            try
            {
                PrintDialog dlgPrint = new PrintDialog();
                dlgPrint.Document = doc;
                if (dlgPrint.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    XmlSerializer xs = new XmlSerializer(typeof(PrinterSettings));
                    StreamWriter sw = new StreamWriter(prnSettingFileName);
                    xs.Serialize(sw, doc.PrinterSettings);
                    sw.Close();
                }
            }
            catch
            {
                MessageBox.Show("Printer Setup Error");
            }
        }

        private void butLoadTempl_Click(object sender, EventArgs e)
        {
            LoadTemplate(true);
        }

        private void LoadTemplate(bool showDlg)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Filter = "QSL template (*.xml)|*.xml";
            //dlg.FileName = @"C:\HAM\DxQSL\DxQSL\bin\debug\ahoj.xml";
            if (showDlg)
            {
                if (dlg.ShowDialog() != DialogResult.OK)
                    return;
                templFile = dlg.FileName;
                UpdateSettings();
                if (templFile != null)
                {
                    int pos = templFile.LastIndexOf('\\');
                    if (pos > 0)
                    {
                        lblTemplFile.Text = templFile.Substring(pos+1);
                    }
                    else
                    lblTemplFile.Text ="???";
                }
            }
            else
            {
                if (templFile.Length < 4)
                    return;
            }
            try
            {
                templ = new XmlDocument();
                //TextReader tr = new StreamReader(templFile, true);
                templ.Load(templFile);
                //templ.Load(tr);
                if (templ.DocumentElement.Name != "dxqsl")
                    throw new Exception("Not a QSL template!");

                templatePath = templFile.Substring(0, templFile.LastIndexOf('\\'));
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error load template");
            }
            LoadPrinterSettings();
        }

        private void butSave_Click(object sender, EventArgs e)
        {
            dataSet1.Tables[1].WriteXml("qsodata.xml");
        }

        private void butLoad_Click(object sender, EventArgs e)
        {
            dataSet1.Tables[1].ReadXml("qsodata.xml");
        }

        private void butClear_Click(object sender, EventArgs e)
        {
            dataSet1.Tables[0].Rows.Clear();
        }


        private void logGrid_DoubleClick(object sender, EventArgs e)
        {
            try
            {
                logGrid.Focus();
                DataGridViewSelectedRowCollection srcs = logGrid.SelectedRows;
                if (srcs.Count != 1)
                    return;
                DataGridViewRow src = srcs[0];
                string call = ((string)src.Cells[0].Value).ToUpper();

                // empty the table when the call in table is different from that new inserted - only out of SWL mode
                if (!swlMode)
                {
                    DataGridViewRow r1 = printGrid.Rows[0];
                    if (r1 != null)
                    {
                        if (call != (string)r1.Cells[0].Value)
                            dataSet1.Tables[0].Rows.Clear();
                    }
                }


                DataRow dr = dataSet1.Tables[0].NewRow();
                dr["Call"] = call;
                dr["Date"] = src.Cells[1].Value;
                dr["MHz"] = src.Cells[2].Value;
                dr["Mode"] = src.Cells[3].Value;
                dr["Rst"] = src.Cells[4].Value;
                dr["Manager"] = src.Cells[5].Value;
                dr["Qth"] = src.Cells[6].Value;
                dataSet1.Tables[0].Rows.Add(dr);
            }
            catch { }
        }

        private void butLoadAdif_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Filter = "Adif log (*.adi,*.adif)|*.adi;*.adif";
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                dataSet1.Tables[1].Clear();
                adifFile = dlg.FileName;
                ReadADIF();
                LoadSettingTemplate();
                if (templFile != null)
                {
                    UpdateSettings();
                }

                if (dlg.FileName != null)
                {
                    int pos = dlg.FileName.LastIndexOf('\\');
                    if (pos > 0)
                    {
                        lblAdifFile.Text = dlg.FileName.Substring(pos + 1);
                    }
                    else
                        lblAdifFile.Text = "???";
                }
            }
        }

        enum readerstate { idle, field, optfield, length, data };
        private void ReadADIF()
        {
            try
            {
                qsocount = 0;
                StreamReader reader = new StreamReader(adifFile);
                readerstate state = readerstate.idle;
                string field = "";
                string datalen = "";
                int length = 0;
                string data = "";
                adifdate = "";
                adiftime = "";
                int c;
                while ((c = reader.Read()) != -1)
                {
                    switch (state)
                    {
                        case readerstate.idle:
                            if (c == '<')
                                state = readerstate.field;
                            break;
                        case readerstate.field:
                            if (c == ':')
                                state = readerstate.length;
                            else if (c == '>')
                            {
                                AdifFlag(field);
                                field = "";
                                state = readerstate.idle;
                            }
                            else
                                field += (char)c;
                            break;
                        case readerstate.length:
                            if (c == '>' || c ==':')
                            {
                                length = int.Parse(datalen);
                                if (c == '>')
                                    state = readerstate.data;
                                else
                                    state = readerstate.optfield;
                            }
                            else
                                datalen += (char)c;
                            break;
                        case readerstate.optfield:
                            if (c == '>')
                                state = readerstate.data;   // optfield data ignored
                            break;
                        case readerstate.data:
                            if (length > 0)
                            {
                                data += (char)c;
                                length--;

                            } 
                            if (length <= 0)
                            {
                                AdifData(field, data);
                                field = "";
                                data = "";
                                datalen = "";
                                state = readerstate.idle;
                            }
                            break;
                    }
                }
            }
            catch
            {
            }
        }

        private void AdifFlag(string flag)
        {
            flag = flag.ToLower();
            if (flag == "eor")
            {
                AdifToDT();
                adifdate = "";
                adiftime = "0000";
                dataSet1.Tables[1].Rows.Add(adifQso);
                qsocount++;
                adifQso = null;
            }
        }

        private void AdifData(string field, string data)
        {
            field = field.ToLower();
            if (adifQso == null)
                adifQso = dataSet1.Tables[1].NewRow();
            switch (field)
            {
                case "call":
                    adifQso["Call"] = data;
                    break;
                case "mode":
                    adifQso["Mode"] = data;
                    break;
                case "qso_date":
                    adifdate = data;
                    break;
                case "time_on":
                    adiftime = data;
                    break;
                case "freq":
                    double freq = double.Parse(data);
                    adifQso["MHz"] = freq;
                    break;
                case "band":
                    if (adifQso.IsNull("MHz"))  // freq has a priority over band
                    {
                        for (int i = 0; i < bandnames.GetUpperBound(0); i++)
                        {
                            if (bandnames[i] == data.ToLower())
                            {
                                adifQso["MHz"] = bandfreq[i];
                                break;
                            }
                        }
                    }
                    break;
                case "rst_sent":
                    adifQso["Rst"] = data;
                    break;
                case "qsl_via":
                    adifQso["Manager"] = data;
                    break;
                case "myqth":
                    adifQso["Qth"] = data;
                    break;
            }
        }
        
        private void AdifToDT()
        {
            try
            {
                if (adifdate == "")
                    adifdate = "20000101";  // in case date not set

                int hour = int.Parse(adiftime.Substring(0, 2));
                int min = int.Parse(adiftime.Substring(2, 2));
                int sec = 0;

                int year = int.Parse(adifdate.Substring(0, 4));
                int month = int.Parse(adifdate.Substring(4, 2));
                int day = int.Parse(adifdate.Substring(6, 2));

                if (adiftime.Length >= 6)
                    sec = int.Parse(adiftime.Substring(4, 2));
              
                adifQso["date"] = new DateTime(year, month, day, hour, min, sec);

            }
            catch { }
        }

        private void txtSearchCall_TextChanged(object sender, EventArgs e)
        {
            string call = txtSearchCall.Text.ToUpper();
            txtSearchCall.SelectionStart = 1000;
            try
            {

                if ((logGrid.SortedColumn == null) || (logGrid.SortedColumn.Index != 0) || (logGrid.SortOrder != SortOrder.Ascending))
                    logGrid.Sort(logGrid.Columns[0], ListSortDirection.Ascending);

                logGrid.CurrentCell = logGrid.Rows[logGrid.Rows.Count - 1].Cells[0];    // to show found row at first line

                foreach (DataGridViewRow row in logGrid.Rows)
                {
                    if (string.Compare(((string)(row.Cells[0].Value)).ToUpper(), call) >= 0)
                    {
                        logGrid.CurrentCell = row.Cells[0];
                        break;
                    }
                }
            }
            catch { }
        }

        private void DxQslDlg_Load(object sender, EventArgs e)
        {
            settings.Reload();
            if (File.Exists(settings.Log1))
            {
                adifFile = settings.Log1;
                dataSet1.Tables[1].Clear();
                ReadADIF();
                int pos = adifFile.LastIndexOf('\\');
                if (pos > 0)
                {
                    lblAdifFile.Text = adifFile.Substring(pos + 1);
                }
                else
                    lblAdifFile.Text = "???";

                LoadSettingTemplate();
            }
        }

        private void UpdateSettings()
        {
            if (settings.Log1 == adifFile)
            {
            }
            else if (settings.Log2 == adifFile)
            {
                settings.Log2 = settings.Log1;
                settings.Tpl2 = settings.Tpl1;
            }
            else if (settings.Log3 == adifFile)
            {
                settings.Log3 = settings.Log1;
                settings.Tpl3 = settings.Tpl1;
            }
            else if (settings.Log4 == adifFile)
            {
                settings.Log4 = settings.Log1;
                settings.Log4 = settings.Tpl1;
            }
            else if (settings.Log5 == adifFile)
            {
                settings.Log5 = settings.Log1;
                settings.Tpl5 = settings.Tpl1;
            }
            else
            {
                settings.Log5 = settings.Log4;
                settings.Log4 = settings.Log3;
                settings.Log3 = settings.Log2;
                settings.Log2 = settings.Log1;

                settings.Tpl5 = settings.Tpl4;
                settings.Tpl4 = settings.Tpl3;
                settings.Tpl3 = settings.Tpl2;
                settings.Tpl2 = settings.Tpl1;
            }
            settings.Log1 = adifFile;
            settings.Tpl1 = templFile;
        }

        void LoadSettingTemplate()
        {
            templFile = null;
            if (settings.Log1 == adifFile)
            {
                templFile = settings.Tpl1;
            }
            else if (settings.Log2 == adifFile)
            {
                templFile = settings.Tpl2;
            }
            else if (settings.Log3 == adifFile)
            {
                templFile = settings.Tpl3;
            }
            else if (settings.Log4 == adifFile)
            {
                templFile = settings.Tpl4;
            }
            else if (settings.Log5 == adifFile)
            {
                templFile = settings.Tpl5;
            }

            if (templFile != null)
            {
                int pos = templFile.LastIndexOf('\\');
                if (pos > 0)
                {
                    lblTemplFile.Text = templFile.Substring(pos + 1);
                    LoadTemplate(false);
                }
                else
                   lblTemplFile.Text = "???";
            }
            else
                lblTemplFile.Text = "???";

        }

        private void DxQslDlg_FormClosing(object sender, FormClosingEventArgs e)
        {
            settings.Save();
        }

    }

 

    abstract class DxBase
    {
        protected float x = 0;
        protected float y = 0;
        protected Color color = Color.Black;
        public abstract void Print(Graphics g, float xoffset, float yoffset);
    }
}
